#
# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Requires Python 2.4+ and Openssl 1.0+
#

import os
import re
import pwd
import shutil
import socket
import array
import struct
import fcntl
import time
import azurelinuxagent.logger as logger
import azurelinuxagent.utils.fileutil as fileutil
import azurelinuxagent.utils.shellutil as shellutil
import azurelinuxagent.utils.textutil as textutil

RulesFiles = [ "/lib/udev/rules.d/75-persistent-net-generator.rules",
               "/etc/udev/rules.d/70-persistent-net.rules" ]

"""
Define distro specific behavior. OSUtil class defines default behavior 
for all distros. Each concrete distro classes could overwrite default behavior
if needed.
"""

class OSUtilError(Exception):
    pass

class DefaultOSUtil(object):

    def __init__(self):
        self.libDir = "/var/lib/waagent"
        self.extLogDir = "/var/log/azure"
        self.dvdMountPoint = "/mnt/cdrom/secure"
        self.ovfenvPathOnDvd = "/mnt/cdrom/secure/ovf-env.xml"
        self.agentPidPath = "/var/run/waagent.pid"
        self.passwdPath = "/etc/shadow"
        self.home = '/home'
        self.sshdConfigPath = '/etc/ssh/sshd_config'
        self.opensslCmd = '/usr/bin/openssl'
        self.configPath = '/etc/waagent.conf'
        self.selinux=None

    def GetLibDir(self):
        return self.libDir

    def GetExtLogDir(self):
        return self.extLogDir

    def GetDvdMountPoint(self):
        return self.dvdMountPoint

    def GetConfigurationPath(self):
        return self.configPath

    def GetOvfEnvPathOnDvd(self):
        return self.ovfenvPathOnDvd

    def GetAgentPidPath(self):
        return self.agentPidPath

    def GetOpensslCmd(self):
        return self.opensslCmd

    def UpdateUserAccount(self, userName, password, expiration=None):
        """
        Update password and ssh key for user account.
        New account will be created if not exists.
        """
        if userName is None:
            raise OSUtilError("User name is empty")

        if self.IsSysUser(userName):
            raise OSUtilError(("User {0} is a system user. "
                               "Will not set passwd.").format(userName))

        userentry = self.GetUserEntry(userName)
        if userentry is None:
            self.CreateUserAccount(userName, expiration)
            
        self.ConfigSudoer(userName, password is None)

    def GetUserEntry(self, userName):
        try:
            return pwd.getpwnam(userName)
        except KeyError:
            return None

    def IsSysUser(self, userName):
        userentry = self.GetUserEntry(userName)
        uidmin = None
        try:
            uidminDef = fileutil.GetLineStartingWith("UID_MIN", "/etc/login.defs")
            if uidminDef is not None:
                uidmin = int(uidminDef.split()[1])
        except IOError as e:
            pass
        if uidmin == None:
            uidmin = 100
        if userentry != None and userentry[2] < uidmin:
            return True
        else:
            return False
   
    def CreateUserAccount(self, userName, expiration=None):
        if expiration is not None:
            cmd = "useradd -m {0} -e {1}".format(userName, expiration)
        else:
            cmd = "useradd -m {0}".format(userName)
        retcode, out = shellutil.RunGetOutput(cmd)
        if retcode != 0:
            raise OSUtilError(("Failed to create user account:{0}, "
                               "retcode:{1}, "
                               "output:{2}").format(userName, retcode, out))

    def ChangePassword(self, userName, password, useSalt=True, saltType=6, 
                       saltLength=10):
        passwdHash = textutil.GetPasswordHash(password, useSalt, saltType, 
                                              saltLength)
        try:
            passwdContent = fileutil.GetFileContents(self.passwdPath)
            passwd = passwdContent.split("\n") 
            newPasswd = filter(lambda x : not x.startswith(userName), passwd)
            newPasswd.append("{0}:{1}:14600::::::".format(userName, passwdHash))
            fileutil.SetFileContents(self.passwdPath, "\n".join(newPasswd))
        except IOError as e:
            raise OSUtilError(("Failed to set password for {0}: {1}"
                               "").format(userName, e))

    def ConfigSudoer(self, userName, nopasswd):
        # for older distros create sudoers.d
        if not os.path.isdir('/etc/sudoers.d/'):
            # create the /etc/sudoers.d/ directory
            os.mkdir('/etc/sudoers.d/')
            # add the include of sudoers.d to the /etc/sudoers
            sudoers = '\n' + '#includedir /etc/sudoers.d/\n'
            fileutil.AppendFileContents('/etc/sudoers', sudoers)
        sudoer = None
        if nopasswd:
            sudoer = "{0} ALL = (ALL) NOPASSWD\n".format(userName)
        else:
            sudoer = "{0} ALL = (ALL) ALL\n".format(userName)
        fileutil.AppendFileContents('/etc/sudoers.d/waagent', sudoer)
        fileutil.ChangeMod('/etc/sudoers.d/waagent', 0440)

    def DeleteRootPassword(self):
        try:
            passwdContent = fileutil.GetFileContents(self.passwdPath)
            passwd = passwdContent.split('\n')
            newPasswd = filter(lambda x : not x.startswith("root:"), passwd)
            newPasswd.insert(0, "root:*LOCK*:14600::::::")
            fileutil.SetFileContents(self.passwdPath, "\n".join(newPasswd))
        except IOError as e:
            raise OSUtilError("Failed to delete root password:{0}".format(e))

    def GetHome(self):
        return self.home
    
    def GetPubKeyFromPrv(self, fileName):
        cmd = "{0} rsa -in {1} -pubout 2>/dev/null".format(self.opensslCmd,
                                                           fileName)
        pub = shellutil.RunGetOutput(cmd)[1]
        return pub

    def GetPubKeyFromCrt(self, fileName):
        cmd = "{0} x509 -in {1} -pubkey -noout".format(self.opensslCmd, 
                                                       fileName)
        pub = shellutil.RunGetOutput(cmd)[1]
        return pub

    def _NormPath(self, filepath):
        home = self.GetHome()
        # Expand HOME variable if present in path
        path = os.path.normpath(filepath.replace("$HOME", home))
        return path
        
    def GetThumbprintFromCrt(self, fileName):
        cmd="{0} x509 -in {1} -fingerprint -noout".format(self.opensslCmd, 
                                                            fileName)
        thumbprint = shellutil.RunGetOutput(cmd)[1]
        thumbprint = thumbprint.rstrip().split('=')[1].replace(':', '').upper()
        return thumbprint
   
    def DeploySshKeyPair(self, userName, thumbprint, path):
        """
        Deploy id_rsa and id_rsa.pub
        """
        path = self._NormPath(path)
        dirPath = os.path.dirname(path)
        fileutil.CreateDir(dirPath, mode=0700, owner=userName)
        libDir = self.GetLibDir()
        prvPath = os.path.join(libDir, thumbprint + '.prv')
        if not os.path.isfile(prvPath):
            logger.Error("Failed to deploy key pair, thumbprint: {0}", 
                         thumbprint)
            return
        shutil.copyfile(prvPath, path)
        pubPath = path + '.pub'
        pub = self.GetPubKeyFromPrv(prvPath)
        fileutil.SetFileContents(pubPath, pub)
        self.SetSelinuxContext(pubPath, 'unconfined_u:object_r:ssh_home_t:s0')
        self.SetSelinuxContext(path, 'unconfined_u:object_r:ssh_home_t:s0')
        os.chmod(path, 0644)
        os.chmod(pubPath, 0600)

    def OpenSslToOpenSsh(self, inputFile, outputFile):
        shellutil.Run("ssh-keygen -i -m PKCS8 -f {0} >> {1}".format(inputFile,
                                                                    outputFile))

    def DeploySshPublicKey(self, userName, thumbprint, path):
        """
        Deploy authorized_key
        """
        path = self._NormPath(path)
        dirPath = os.path.dirname(path)
        fileutil.CreateDir(dirPath, mode=0700, owner=userName)
        libDir = self.GetLibDir()
        crtPath = os.path.join(libDir, thumbprint + '.crt')
        if not os.path.isfile(crtPath):
            logger.Error("Failed to deploy public key, thumbprint: {0}", 
                         thumbprint)
            return
        pubPath = os.path.join(libDir, thumbprint + '.pub')
        pub = self.GetPubKeyFromCrt(crtPath)
        fileutil.SetFileContents(pubPath, pub)
        self.SetSelinuxContext(pubPath, 'unconfined_u:object_r:ssh_home_t:s0')
        self.OpenSslToOpenSsh(pubPath, path)
        self.SetSelinuxContext(path, 'unconfined_u:object_r:ssh_home_t:s0')
        fileutil.ChangeOwner(path, userName)
        fileutil.ChangeMod(path, 0644)
        fileutil.ChangeMod(pubPath, 0600)
            
    def IsSelinuxSystem(self):
        """
        Checks and sets self.selinux = True if SELinux is available on system.
        """
        if self.selinux == None:
            if shellutil.Run("which getenforce", chk_err=False) == 0:
                self.selinux = True
            else:
                self.selinux = False
        return self.selinux
    
    def IsSelinuxRunning(self):
        """
        Calls shell command 'getenforce' and returns True if 'Enforcing'.
        """
        if self.IsSelinuxSystem():
            output = shellutil.RunGetOutput("getenforce")[1]
            return output.startswith("Enforcing")
        else:
            return False
        
    def SetSelinuxEnforce(self, state):
        """
        Calls shell command 'setenforce' with 'state' 
        and returns resulting exit code.
        """
        if self.IsSelinuxSystem():
            if state: s = '1'
            else: s='0'
            return shellutil.Run("setenforce "+s)

    def SetSelinuxContext(self, path, cn):
        """
        Calls shell 'chcon' with 'path' and 'cn' context.
        Returns exit result.
        """
        if self.IsSelinuxSystem():
            return shellutil.Run('chcon ' + cn + ' ' + path)
   
    def GetSshdConfigPath(self):
        return self.sshdConfigPath

    def SetSshClientAliveInterval(self):
        configPath = self.GetSshdConfigPath()
        config = fileutil.GetFileContents(configPath).split("\n")
        textutil.SetSshConfig(config, "ClientAliveInterval", "180")
        fileutil.ReplaceFileContentsAtomic(configPath, '\n'.join(config))
        logger.Info("Configured SSH client probing to keep connections alive.")
   
    def ConfigSshd(self, disablePassword):
        option = "no" if disablePassword else "yes"
        configPath = self.GetSshdConfigPath()
        config = fileutil.GetFileContents(configPath).split("\n")
        textutil.SetSshConfig(config, "PasswordAuthentication", option)
        textutil.SetSshConfig(config, "ChallengeResponseAuthentication", option)
        fileutil.ReplaceFileContentsAtomic(configPath, "\n".join(config))
        logger.Info("Disabled SSH password-based authentication methods.")


    def GetDvdDevice(self, devDir='/dev'):
        patten=r'(sr[0-9]|hd[c-z]|cdrom[0-9])'
        for dvd in [re.match(patten, dev) for dev in os.listdir(devDir)]:
            if dvd is not None:
                return "/dev/{0}".format(dvd.group(0))
        raise OSUtilError("Failed to get dvd device")

    def MountDvd(self, maxRetry=6, chk_err=True):
        dvd = self.GetDvdDevice()
        mountPoint = self.GetDvdMountPoint()
        mountlist = shellutil.RunGetOutput("mount")[1]
        existing = self.GetMountPoint(mountlist, dvd)
        if existing is not None: #Already mounted
            logger.Info("{0} is already mounted at {1}", dvd, existing)
            return
        if not os.path.isdir(mountPoint):
            os.makedirs(mountPoint)
        
        for retry in range(0, maxRetry):
            retcode = self.Mount(dvd, mountPoint, option="-o ro -t iso9660,udf", 
                                 chk_err=chk_err)
            if retcode == 0:
                logger.Info("Successfully mounted dvd")
                return
            if retry < maxRetry - 1:
                logger.Warn("Mount dvd failed: retry={0}, ret={1}", retry, 
                            retcode)
                time.sleep(5)
        if chk_err:
            raise OSUtilError("Failed to mount dvd.")

    def UmountDvd(self, chk_err=True):
        mountPoint = self.GetDvdMountPoint()
        retcode = self.Umount(mountPoint, chk_err=chk_err)
        if chk_err and retcode != 0:
            raise OSUtilError("Failed to umount dvd.")

    def LoadAtapiixModule(self):
        if self.IsAtaPiixModuleLoaded():
            return
        ret, kernVersion = shellutil.RunGetOutput("uname -r")
        if ret != 0:
            raise Exception("Failed to call uname -r")
        modulePath = os.path.join('/lib/modules', 
                                  kernVersion.strip('\n'),
                                  'kernel/drivers/ata/ata_piix.ko')
        if not os.path.isfile(modulePath):
            raise Exception("Can't find module file:{0}".format(modulePath))

        ret, output = shellutil.RunGetOutput("insmod " + modulePath)
        if ret != 0:
            raise Exception("Error calling insmod for ATAPI CD-ROM driver")
        if not self.IsAtaPiixModuleLoaded(maxRetry=3):
            raise Exception("Failed to load ATAPI CD-ROM driver") 

    def IsAtaPiixModuleLoaded(self, maxRetry=1):
        for retry in range(0, maxRetry):
            ret = shellutil.Run("lsmod | grep ata_piix", chk_err=False)
            if ret == 0:
                logger.Info("Module driver for ATAPI CD-ROM is already present.")
                return True
            if retry < maxRetry - 1:
                time.sleep(1)
        return False
 
    def Mount(self, dvd, mountPoint, option="", chk_err=True):
        cmd = "mount {0} {1} {2}".format(dvd, option,  mountPoint)
        return shellutil.RunGetOutput(cmd, chk_err)[0]

    def Umount(self, mountPoint, chk_err=True):
        return shellutil.Run("umount {0}".format(mountPoint), chk_err=chk_err)

    def OpenPortForDhcp(self):
        #Open DHCP port if iptables is enabled.
        # We supress error logging on error.
        shellutil.Run("iptables -D INPUT -p udp --dport 68 -j ACCEPT",
                      chk_err=False)      
        shellutil.Run("iptables -I INPUT -p udp --dport 68 -j ACCEPT",
                      chk_err=False)

    def GenerateTransportCert(self):
        """
        Create ssl certificate for https communication with endpoint server.
        """
        cmd = ("{0} req -x509 -nodes -subj /CN=LinuxTransport -days 32768 "
               "-newkey rsa:2048 -keyout TransportPrivate.pem "
               "-out TransportCert.pem").format(self.opensslCmd)
        shellutil.Run(cmd)

    def RemoveRulesFiles(self, rulesFiles=RulesFiles):
        libDir = self.GetLibDir()
        for src in rulesFiles:
            fileName = fileutil.GetLastPathElement(src)
            dest = os.path.join(libDir, fileName)
            if os.path.isfile(dest):
                os.remove(dest)
            if os.path.isfile(src):
                logger.Warn("Move rules file {0} to {1}", fileName, dest)
                shutil.move(src, dest)

    def RestoreRulesFiles(self, rulesFiles=RulesFiles):
        libDir = self.GetLibDir()
        for dest in rulesFiles:
            fileName = fileutil.GetLastPathElement(dest)
            src = os.path.join(libDir, fileName)
            if os.path.isfile(dest):
                continue
            if os.path.isfile(src):
                logger.Warn("Move rules file {0} to {1}", fileName, dest)
                shutil.move(src, dest)

    def GetMacAddress(self):
        """
        Convienience function, returns mac addr bound to
        first non-loobback interface.
        """
        ifname=''
        while len(ifname) < 2 :
            ifname=self.GetFirstActiveNetworkInterfaceNonLoopback()[0]
        addr = self.GetInterfaceMac(ifname)        
        return textutil.HexStringToByteArray(addr)

    def GetInterfaceMac(self, ifname):
        """
        Return the mac-address bound to the socket.
        """
        sock = socket.socket(socket.AF_INET, 
                             socket.SOCK_DGRAM, 
                             socket.IPPROTO_UDP)
        param = struct.pack('256s', (ifname[:15]+('\0'*241)).encode('latin-1'))
        info = fcntl.ioctl(sock.fileno(), 0x8927, param)
        return ''.join(['%02X' % textutil.Ord(char) for char in info[18:24]])

    def GetFirstActiveNetworkInterfaceNonLoopback(self):
        """
        Return the interface name, and ip addr of the
        first active non-loopback interface.
        """
        iface=''
        expected=16 # how many devices should I expect...
        struct_size=40 # for 64bit the size is 40 bytes
        sock = socket.socket(socket.AF_INET, 
                             socket.SOCK_DGRAM, 
                             socket.IPPROTO_UDP)
        buff=array.array('B', b'\0' * (expected * struct_size))
        param = struct.pack('iL', 
                            expected*struct_size, 
                            buff.buffer_info()[0])
        ret = fcntl.ioctl(sock.fileno(), 0x8912, param)
        retsize=(struct.unpack('iL', ret)[0])
        if retsize == (expected * struct_size):
            logger.Warn(('SIOCGIFCONF returned more than {0} up '
                         'network interfaces.'), expected)
        sock = buff.tostring()
        for i in range(0, struct_size * expected, struct_size):
            iface=sock[i:i+16].split(b'\0', 1)[0]
            if iface == b'lo':
                continue
            else:
                break
        return iface.decode('latin-1'), socket.inet_ntoa(sock[i+20:i+24])

    def IsMissingDefaultRoute(self):
        routes = shellutil.RunGetOutput("route -n")[1]
        for route in routes:
            if route.startswith("0.0.0.0 ") or route.startswith("default "):
               return False 
        return True

    def GetInterfaceName(self):
        return self.GetFirstActiveNetworkInterfaceNonLoopback()[0]
    
    def GetIpv4Address(self):
        return self.GetFirstActiveNetworkInterfaceNonLoopback()[1]
    
    def SetBroadcastRouteForDhcp(self, ifname):
        return shellutil.Run("route add 255.255.255.255 dev {0}".format(ifname),
                             chk_err=False)

    def RemoveBroadcastRouteForDhcp(self, ifname):
        shellutil.Run("route del 255.255.255.255 dev {0}".format(ifname), 
                      chk_err=False)

    def IsDhcpEnabled(self):
        return False

    def StopDhcpService(self):
        pass

    def StartDhcpService(self):
        pass

    def StartNetwork(self):
        pass

    def StartAgentService(self):
        pass

    def StopAgentService(self):
        pass

    def RegisterAgentService(self):
        pass

    def UnregisterAgentService(self):
        pass

    def RestartSshService(self):
        pass
    
    def RouteAdd(self, net, mask, gateway):
        """
        Add specified route using /sbin/route add -net.
        """
        cmd = ("/sbin/route add -net "
               "{0} netmask {1} gw {2}").format(net, mask, gateway)
        return shellutil.Run(cmd, chk_err=False)

    def GetDhcpProcessId(self):
        ret= shellutil.RunGetOutput("pidof dhclient")
        return ret[1] if ret[0] == 0 else None

    def SetHostname(self, hostname):
        fileutil.SetFileContents('/etc/hostname', hostname)
        shellutil.Run("hostname {0}".format(hostname), chk_err=False)

    def SetDhcpHostname(self, hostname):
        autoSend = r'^[^#]*?send\s*host-name.*?(<hostname>|gethostname[(,)])'
        dhclientFiles = ['/etc/dhcp/dhclient.conf', '/etc/dhcp3/dhclient.conf']
        for confFile in dhclientFiles:
            if not os.path.isfile(confFile):
                continue
            if fileutil.FindStringInFile(confFile, autoSend):
                #Return if auto send host-name is configured
                return
            fileutil.UpdateConfigFile(confFile,
                                      'send host-name',
                                      'send host-name {0}'.format(hostname))

    def RestartInterface(self, ifname):
        shellutil.Run("ifdown {0} && ifup {1}".format(ifname, ifname))

    def PublishHostname(self, hostname):
        self.SetDhcpHostname(hostname)
        ifname = self.GetInterfaceName()
        self.RestartInterface(ifname)

    def SetScsiDiskTimeout(self, timeout):
        for dev in os.listdir("/sys/block"):
            if dev.startswith('sd'):
                self.SetBlockDeviceTimeout(dev, timeout)

    def SetBlockDeviceTimeout(self, dev, timeout):
        if dev is not None and timeout is not None:
            filePath = "/sys/block/{0}/device/timeout".format(dev)
            content = fileutil.GetFileContents(filePath)
            original = content.splitlines()[0].rstrip()
            if original != timeout:
                fileutil.SetFileContents(filePath, timeout)
                logger.Info("Set block dev timeout: {0} with timeout: {1}",
                            dev, timeout)

    def GetMountPoint(self, mountlist, device):
        """
        Example of mountlist:
            /dev/sda1 on / type ext4 (rw)
            proc on /proc type proc (rw)
            sysfs on /sys type sysfs (rw)
            devpts on /dev/pts type devpts (rw,gid=5,mode=620)
            tmpfs on /dev/shm type tmpfs 
            (rw,rootcontext="system_u:object_r:tmpfs_t:s0")
            none on /proc/sys/fs/binfmt_misc type binfmt_misc (rw)
            /dev/sdb1 on /mnt/resource type ext4 (rw)
        """
        if (mountlist and device):
            for entry in mountlist.split('\n'):
                if(re.search(device, entry)):
                    tokens = entry.split()
                    #Return the 3rd column of this line
                    return tokens[2] if len(tokens) > 2 else None
        return None
    
    def DeviceForIdePort(self, n):
        """
        Return device name attached to ide port 'n'.
        """
        if n > 3:
            return None
        g0 = "00000000"
        if n > 1:
            g0 = "00000001"
            n = n - 2
        device = None
        path = "/sys/bus/vmbus/devices/"
        for vmbus in os.listdir(path):
            deviceid = fileutil.GetFileContents(os.path.join(path, vmbus, 
                                                             "device_id"))
            guid = deviceid.lstrip('{').split('-')
            if guid[0] == g0 and guid[1] == "000" + str(n):
                for root, dirs, files in os.walk(path + vmbus):
                    if root.endswith("/block"):
                        device = dirs[0]
                        break
                    else : #older distros
                        for d in dirs:
                            if ':' in d and "block" == d.split(':')[0]:
                                device = d.split(':')[1]
                                break
                break
        return device

    def DeleteAccount(self, userName):
        if self.IsSysUser(userName):
            logger.Error("{0} is a system user. Will not delete it.", userName)
        shellutil.Run("> /var/run/utmp")
        shellutil.Run("userdel -f -r " + userName)
        #Remove user from suders
        if os.path.isfile("/etc/suders.d/waagent"):
            try:
                content = fileutil.GetFileContents("/etc/sudoers.d/waagent")
                sudoers = content.split("\n")
                sudoers = filter(lambda x : userName not in x, sudoers)
                fileutil.SetFileContents("/etc/sudoers.d/waagent", 
                                         "\n".join(sudoers))
            except IOError as e:
                raise OSUtilError("Failed to remove sudoer: {0}".format(e))
  
    def TranslateCustomData(self, data):
        return data

    def GetTotalMemory(self):
        cmd = "grep MemTotal /proc/meminfo |awk '{print $2}'"
        ret = shellutil.RunGetOutput(cmd)
        if ret[0] == 0:
            return int(ret[1])/1024
        else:
            raise OSUtilError("Failed to get total memory: {0}".format(ret[1]))

    def GetProcessorCores(self):
        ret = shellutil.RunGetOutput("grep 'processor.*:' /proc/cpuinfo |wc -l")
        if ret[0] == 0:
            return int(ret[1])
        else:
            raise OSUtilError("Failed to get procerssor cores")

OSUtil = DefaultOSUtil
